;*****************************************************************************
;				NICST TOOL
;
;  Uses polynominal fits, linear (ifit=0),  quadratic (ifit=1) 
;  and cubic equation (ifit=2), relating dn to radiance. 
;
;  Input:
;  1) Input file.
;
;  Output:
;  1) ncollects
;  2) rg[band,HAM,detector,subsample,a0 a1 a2 a3,fit,rsr]
;  3) rg_1[band,HAM,detector,subsample,scans,a0 a1 a2 a3,fit,rsr] --
;  cross calibrated.
;  4) rg_2[band,HAM,detector,subsample,scans,a0 a1 a2 a3,fit,rsr] --
;  not cross calibrated.
;  5) rg_1_error[band,HAM,detector,subsample,scans,a0 a1 a2 a3,fit,rsr]
;  6) dL_fit[collects,band,HAM,detector,subsample,scans,fit,rsr]
;  7) rrcu[band,HAM,detector,subsample,rsr,0=warm-up 1=cool-down]
;  8) rrnl[band,HAM,detector,subsample,rsr,0=warm-up 1=cool-down]
;  9) F_1[bands,HAM,detector,subsamples,fit,rsr] -- cross calibration factor.
;  10) F_2[collects,bands,HAM,detector,subsamples,scans/2,fit,rsr] --
;  cross caibration factor.
;  11) dn_x[collects,bands,HAM,detector,subsamples,scans/2]
;  12) rad_OBC_scan_x[collects,bands,HAM,detector,subsamples,scans/2,rsr]
;  
;  v1.0 - 03/27/2008 - Jeff McIntire
;  v2.0 - 05/27/2008 - Jeff McIntire
;  v3.0 - 03/16/2009 - Jeff McIntire
;    -- added error estimates and cross calibration
;*****************************************************************************

PRO RC5_OBC_fit, infile

;*****************************************************************************
;  Read in input file, sav TEB file, and make some definitions.
;*****************************************************************************

nlines=File_Lines(infile)
ab=''
header=strarr(1)
inpath=strarr(1)		; Input path.
outpath=strarr(1)		; Output path.
rvs_in=strarr(1)		; RVS .sav file.
param_in=strarr(1)		; EDU coeff .sav file.
rsrpath=strarr(1)		; RSR .sav file.
maxscan=intarr(1)               ; # of scans

openr,lun,infile,/Get_Lun	
readf,lun,header	
readf,lun,inpath
readf,lun,outpath
readf,lun,rvs_in
readf,lun,param_in
readf,lun,rsrpath
readf,lun,maxscan
Free_Lun,lun

maxscans=maxscan[0]

a2=strsplit(outpath,'/',/Extract)
a3=N_ELEMENTS(a2)

restore,'general.TEB.2.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.BCS.rg.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.dn.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rad.sav'
restore,outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temp.sav'

outfile=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.sav'

;*****************************************************************************
;  Definitions.
;*****************************************************************************

G=findgen(ncollects)

x1=dblarr(ncollects)
x2=dblarr(ncollects)
z1=dblarr(ncollects)
z2=dblarr(ncollects)

rg_2=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,4,12,2)
rg_1_error=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,4,12,2)
rg=dblarr(nbands,nham,maxdet,maxsubsets,2,2)
dL_fit=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxscans,12,2)

F_2=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxscans,12,2)
F_1=dblarr(nbands,nham,maxdet,maxsubsets,12,2)

dn_x=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxscans)
rad_OBC_scan_x=dblarr(ncollects,nbands,nham,maxdet,maxsubsets,maxscans,2)
mean_dL_fit=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,2)
abs_max_dL_fit=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,2)
stddev_dL_fit=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,2)
rrcu_scan=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,2,2)
rrnl_scan=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,2,2)
rrcu=dblarr(nbands,nham,maxdet,maxsubsets,2,2)
rrnl=dblarr(nbands,nham,maxdet,maxsubsets,2,2)

;*****************************************************************************
;  Reform the dn and radiance to get rid of extra zeros.
;*****************************************************************************

FOR b=0, nbands-1 DO BEGIN
    FOR is=0, subsets[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                FOR col=0, ncollects-1 DO BEGIN
                    index1=WHERE(dn_col[col,b,ih,id,is,1,*,2] NE 0, arr1)
                    dn_x[col,b,ih,id,is,0:49]=dn_col[col,b,ih,id,is,1,index1,2]
                    FOR rsr=0, 1 DO BEGIN
                        index2=WHERE(abs(rad_OBC_scan[col,b,ih,id,is,*,rsr]) GT 0.01, arr2)
                        rad_OBC_scan_x[col,b,ih,id,is,0:49,rsr]=rad_OBC_scan[col,b,ih,id,is,index2,rsr]
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Fitting using POLY_FIT (Linear and Quadratic and Cubic).
;*****************************************************************************

FOR rsr=0, 1 DO BEGIN
    FOR iscan=0, maxscans/2-1 DO BEGIN
        FOR b=0, nbands-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR ih=0, nham-1 DO BEGIN
                    max_col=WHERE(T_obc[ih,*] EQ max(T_obc[ih,*]))
                    warm=WHERE(G[0:ncollects-1] LE max_col[0])
                    cool=WHERE(G[0:ncollects-1] GT max_col[0])

                    FOR id=0, detectors[b]-1 DO BEGIN
                        
                        x1=dn_x[warm,b,ih,id,is,iscan]
                        z1=rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr]
                        x2=dn_x[cool,b,ih,id,is,iscan]
                        z2=rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr]

;*****************************************************************************
;  Conduct linear, quadratic, and cubic poly_fits to dn vs deltaL (ifit 0-2) 
;  and deltaL vs dn (ifit 3-5). These fits are for the warm up.
;*****************************************************************************

                        result1=POLY_FIT(x1,z1,1,SIGMA=error)
                        result2=POLY_FIT(x1,z1,2,SIGMA=error)
                        result3=POLY_FIT(x1,z1,3,SIGMA=error)
                        result1_1=POLY_FIT(z1,x1,1,SIGMA=error)
                        result2_1=POLY_FIT(z1,x1,2,SIGMA=error)
                        result3_1=POLY_FIT(z1,x1,3,SIGMA=error)
                        
                        rg_2[b,ih,id,is,iscan,0:1,0,rsr]=result1[0:1]
                        rg_2[b,ih,id,is,iscan,0:2,1,rsr]=result2[0:2]
                        rg_2[b,ih,id,is,iscan,*,2,rsr]=result3[*]
                        rg_2[b,ih,id,is,iscan,0:1,3,rsr]=result1_1[0:1]
                        rg_2[b,ih,id,is,iscan,0:2,4,rsr]=result2_1[0:2]
                        rg_2[b,ih,id,is,iscan,*,5,rsr]=result3_1[*]

                        rg_1_error[b,ih,id,is,iscan,0:1,0,rsr]=error[0:1]
                        rg_1_error[b,ih,id,is,iscan,0:2,1,rsr]=error[0:2]
                        rg_1_error[b,ih,id,is,iscan,*,2,rsr]=error[*]
                        rg_1_error[b,ih,id,is,iscan,0:1,3,rsr]=error[0:1]
                        rg_1_error[b,ih,id,is,iscan,0:2,4,rsr]=error[0:2]
                        rg_1_error[b,ih,id,is,iscan,*,5,rsr]=error[*]

;*****************************************************************************
;  Get the fitting residuals for the warm-up.
;*****************************************************************************

                        dL_fit[warm,b,ih,id,is,iscan,0,rsr]= $
                          (rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr]-result1[0]-result1[1]*dn_x[warm,b,ih,id,is,iscan]) $
                          /(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr])
                        dL_fit[warm,b,ih,id,is,iscan,1,rsr]= $
                          (rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr]-result2[0]-result2[1]*dn_x[warm,b,ih,id,is,iscan] $
                           -result2[2]*dn_x[warm,b,ih,id,is,iscan]^2)/(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr])
                        dL_fit[warm,b,ih,id,is,iscan,2,rsr]= $
                          (rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr]-result3[0]-result3[1]*dn_x[warm,b,ih,id,is,iscan] $
                           -result3[2]*dn_x[warm,b,ih,id,is,iscan]^2-result3[3]*dn_x[warm,b,ih,id,is,iscan]^3) $
                          /(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr])
                        
                        dL_fit[warm,b,ih,id,is,iscan,3,rsr]= $
                          (dn_x[warm,b,ih,id,is,iscan]-result1_1[0]-result1_1[1]*(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr])) $
                          /(dn_x[warm,b,ih,id,is,iscan])
                        dL_fit[warm,b,ih,id,is,iscan,4,rsr]= $
                          (dn_x[warm,b,ih,id,is,iscan]-result2_1[0]-result2_1[1]*(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr]) $
                           -result2_1[2]*(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr])^2)/(dn_x[warm,b,ih,id,is,iscan])
                        dL_fit[warm,b,ih,id,is,iscan,5,rsr]= $
                          (dn_x[warm,b,ih,id,is,iscan]-result3_1[0]-result3_1[1]*(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr]) $
                           -result3_1[2]*(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr])^2 $
                           -result3_1[3]*(rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr])^3) $
                          /(dn_x[warm,b,ih,id,is,iscan])

;*****************************************************************************
;  Conduct linear, quadratic, and cubic poly_fits to dn vs deltaL (ifit 6-8) 
;  and deltaL vs dn (ifit 9-11). These fits are for the cool down.
;*****************************************************************************

                        result4=POLY_FIT(x2,z2,1,SIGMA=error)
                        result5=POLY_FIT(x2,z2,2,SIGMA=error)
                        result6=POLY_FIT(x2,z2,3,SIGMA=error)
                        result4_1=POLY_FIT(z2,x2,1,SIGMA=error)
                        result5_1=POLY_FIT(z2,x2,2,SIGMA=error)
                        result6_1=POLY_FIT(z2,x2,3,SIGMA=error)
                        
                        rg_2[b,ih,id,is,iscan,0:1,6,rsr]=result4[0:1]
                        rg_2[b,ih,id,is,iscan,0:2,7,rsr]=result5[0:2]
                        rg_2[b,ih,id,is,iscan,*,8,rsr]=result6[*]
                        rg_2[b,ih,id,is,iscan,0:1,9,rsr]=result4_1[0:1]
                        rg_2[b,ih,id,is,iscan,0:2,10,rsr]=result5_1[0:2]
                        rg_2[b,ih,id,is,iscan,*,11,rsr]=result6_1[*]

                        rg_1_error[b,ih,id,is,iscan,0:1,6,rsr]=error[0:1]
                        rg_1_error[b,ih,id,is,iscan,0:2,7,rsr]=error[0:2]
                        rg_1_error[b,ih,id,is,iscan,*,8,rsr]=error[*]
                        rg_1_error[b,ih,id,is,iscan,0:1,9,rsr]=error[0:1]
                        rg_1_error[b,ih,id,is,iscan,0:2,10,rsr]=error[0:2]
                        rg_1_error[b,ih,id,is,iscan,*,11,rsr]=error[*]

;*****************************************************************************
;  Get the fitting residuals for the cool-down.
;*****************************************************************************

                        dL_fit[cool,b,ih,id,is,iscan,6,rsr]= $
                          (rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr]-result4[0]-result4[1]*dn_x[cool,b,ih,id,is,iscan]) $
                          /(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr])
                        dL_fit[cool,b,ih,id,is,iscan,7,rsr]= $
                          (rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr]-result5[0]-result5[1]*dn_x[cool,b,ih,id,is,iscan] $
                           -result5[2]*dn_x[cool,b,ih,id,is,iscan]^2)/(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr])
                        dL_fit[cool,b,ih,id,is,iscan,8,rsr]= $
                          (rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr]-result6[0]-result6[1]*dn_x[cool,b,ih,id,is,iscan] $
                           -result6[2]*dn_x[cool,b,ih,id,is,iscan]^2-result6[3]*dn_x[cool,b,ih,id,is,iscan]^3) $
                          /(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr])
                        
                        dL_fit[cool,b,ih,id,is,iscan,9,rsr]= $
                          (dn_x[cool,b,ih,id,is,iscan]-result4_1[0]-result4_1[1]*(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr])) $
                          /(dn_x[cool,b,ih,id,is,iscan])
                        dL_fit[cool,b,ih,id,is,iscan,10,rsr]= $
                          (dn_x[cool,b,ih,id,is,iscan]-result5_1[0]-result5_1[1]*(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr]) $
                           -result5_1[2]*(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr])^2)/(dn_x[cool,b,ih,id,is,iscan])
                        dL_fit[cool,b,ih,id,is,iscan,11,rsr]= $
                          (dn_x[cool,b,ih,id,is,iscan]-result6_1[0]-result6_1[1]*(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr]) $
                           -result6_1[2]*(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr])^2 $
                           -result6_1[3]*(rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr])^3) $
                          /(dn_x[cool,b,ih,id,is,iscan])

;*****************************************************************************
;  Get the radiometric response characterization uncertainty
;  (quadratic fit only) and radiometric response non-linearity (linear fit only).
;*****************************************************************************

                        mean_dL_fit[b,ih,id,is,iscan,rsr]=mean(dL_fit[warm,b,ih,id,is,iscan,1,rsr])
                        stddev_dL_fit[b,ih,id,is,iscan,rsr]=stddev(dL_fit[warm,b,ih,id,is,iscan,1,rsr])
                        rrcu_scan[b,ih,id,is,iscan,rsr,0]=sqrt(mean_dL_fit[b,ih,id,is,iscan,rsr]^2+stddev_dL_fit[b,ih,id,is,iscan,rsr]^2)
                        
                        mean_dL_fit[b,ih,id,is,iscan,rsr]=mean(dL_fit[cool,b,ih,id,is,iscan,7,rsr])
                        stddev_dL_fit[b,ih,id,is,iscan,rsr]=stddev(dL_fit[cool,b,ih,id,is,iscan,7,rsr])
                        rrcu_scan[b,ih,id,is,iscan,rsr,1]=sqrt(mean_dL_fit[b,ih,id,is,iscan,rsr]^2+stddev_dL_fit[b,ih,id,is,iscan,rsr]^2)

                        abs_max_dL_fit[b,ih,id,is,iscan,rsr]=max(abs(dL_fit[warm,b,ih,id,is,iscan,3,rsr]*dn_x[warm,b,ih,id,is,iscan]))
                        rrnl_scan[b,ih,id,is,iscan,rsr,0]=abs_max_dL_fit[b,ih,id,is,iscan,rsr]/(L_max[b,0]*rg_2[b,ih,id,is,iscan,1,4,rsr])
                        abs_max_dL_fit[b,ih,id,is,iscan,rsr]=max(abs(dL_fit[cool,b,ih,id,is,iscan,9,rsr]*dn_x[cool,b,ih,id,is,iscan]))
                        rrnl_scan[b,ih,id,is,iscan,rsr,1]=abs_max_dL_fit[b,ih,id,is,iscan,rsr]/(L_max[b,0]*rg_2[b,ih,id,is,iscan,1,10,rsr])
                    ENDFOR
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Calculate cross calibration factor.
;*****************************************************************************

FOR rsr=0, 1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    FOR iscan=0, maxscans/2-1 DO BEGIN
                        F_2[warm,b,ih,id,is,iscan,0,rsr]=rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr] $
                          /(rg_1[b,ih,id,is,0,0,rsr] $
                            +rg_1[b,ih,id,is,1,0,rsr]*dn_x[warm,b,ih,id,is,iscan])
                        F_2[warm,b,ih,id,is,iscan,1,rsr]=rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr] $
                          /(rg_1[b,ih,id,is,0,1,rsr] $
                            +rg_1[b,ih,id,is,1,1,rsr]*dn_x[warm,b,ih,id,is,iscan] $
                            +rg_1[b,ih,id,is,2,1,rsr]*dn_x[warm,b,ih,id,is,iscan]^2)
                        F_2[warm,b,ih,id,is,iscan,2,rsr]=rad_OBC_scan_x[warm,b,ih,id,is,iscan,rsr] $
                          /(rg_1[b,ih,id,is,0,2,rsr] $
                            +rg_1[b,ih,id,is,1,2,rsr]*dn_x[warm,b,ih,id,is,iscan] $
                            +rg_1[b,ih,id,is,2,2,rsr]*dn_x[warm,b,ih,id,is,iscan]^2 $
                            +rg_1[b,ih,id,is,3,2,rsr]*dn_x[warm,b,ih,id,is,iscan]^3)

                        F_2[cool,b,ih,id,is,iscan,6,rsr]=rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr] $
                          /(rg_1[b,ih,id,is,0,0,rsr] $
                            +rg_1[b,ih,id,is,1,0,rsr]*dn_x[cool,b,ih,id,is,iscan])
                        F_2[cool,b,ih,id,is,iscan,7,rsr]=rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr] $
                          /(rg_1[b,ih,id,is,0,1,rsr] $
                            +rg_1[b,ih,id,is,1,1,rsr]*dn_x[cool,b,ih,id,is,iscan] $
                            +rg_1[b,ih,id,is,2,1,rsr]*dn_x[cool,b,ih,id,is,iscan]^2)
                        F_2[cool,b,ih,id,is,iscan,8,rsr]=rad_OBC_scan_x[cool,b,ih,id,is,iscan,rsr] $
                          /(rg_1[b,ih,id,is,0,2,rsr] $
                            +rg_1[b,ih,id,is,1,2,rsr]*dn_x[cool,b,ih,id,is,iscan] $
                            +rg_1[b,ih,id,is,2,2,rsr]*dn_x[cool,b,ih,id,is,iscan]^2 $
                            +rg_1[b,ih,id,is,3,2,rsr]*dn_x[cool,b,ih,id,is,iscan]^3)
                    ENDFOR
                    F_1[b,ih,id,is,0,rsr]=sig_mean(F_2[warm,b,ih,id,is,0:maxscans/2-1,0,rsr],3,1)
                    F_1[b,ih,id,is,1,rsr]=sig_mean(F_2[warm,b,ih,id,is,0:maxscans/2-1,1,rsr],3,1)
                    F_1[b,ih,id,is,2,rsr]=sig_mean(F_2[warm,b,ih,id,is,0:maxscans/2-1,2,rsr],3,1)
                    
                    F_1[b,ih,id,is,6,rsr]=sig_mean(F_2[cool,b,ih,id,is,0:maxscans/2-1,6,rsr],3,1)
                    F_1[b,ih,id,is,7,rsr]=sig_mean(F_2[cool,b,ih,id,is,0:maxscans/2-1,7,rsr],3,1)
                    F_1[b,ih,id,is,8,rsr]=sig_mean(F_2[cool,b,ih,id,is,0:maxscans/2-1,8,rsr],3,1)
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

;*****************************************************************************
;  Average over scans for RRNL and RRCU, print band averaged gains.
;  Cross calibrate form the BCS to the OBC.
;*****************************************************************************

rg_1=dblarr(nbands,nham,maxdet,maxsubsets,maxscans,4,12,2)
FOR rsr=0, 1 DO BEGIN
    FOR b=0, nbands-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    FOR iscan=0, maxscans/2-1 DO BEGIN
                        rg_1[b,ih,id,is,iscan,rsr,0]=rg_2[b,ih,id,is,iscan,1,1,rsr]*mean(F_2[warm,b,ih,id,is,iscan,1,rsr])
                        rg_1[b,ih,id,is,iscan,rsr,1]=rg_2[b,ih,id,is,iscan,1,7,rsr]*mean(F_2[cool,b,ih,id,is,iscan,7,rsr])
                    ENDFOR
                    rg[b,ih,id,is,rsr,0]=sig_mean(rg_1[b,ih,id,is,0:maxscans/2-1,rsr,0],3,1)
                    rg[b,ih,id,is,rsr,1]=sig_mean(rg_1[b,ih,id,is,0:maxscans/2-1,rsr,1],3,1)
                    
                    rrcu[b,ih,id,is,rsr,0]=sig_mean(rrcu_scan[b,ih,id,is,0:maxscans/2-1,rsr,0],3,1)
                    rrcu[b,ih,id,is,rsr,1]=sig_mean(rrcu_scan[b,ih,id,is,0:maxscans/2-1,rsr,1],3,1)
                    rrnl[b,ih,id,is,rsr,0]=sig_mean(rrnl_scan[b,ih,id,is,0:maxscans/2-1,rsr,0],3,1)
                    rrnl[b,ih,id,is,rsr,1]=sig_mean(rrnl_scan[b,ih,id,is,0:maxscans/2-1,rsr,1],3,1)
                ENDFOR
                print,'Warm-up   ',band_name[b],ih,is,rsr,1/mean(rg[b,ih,1:detectors[b]-1,is,rsr,0]),stddev(1/rg[b,ih,1:detectors[b]-1,is,rsr,0])
                print,'Cool-down ',band_name[b],ih,is,rsr,1/mean(rg[b,ih,1:detectors[b]-1,is,rsr,1]),stddev(1/rg[b,ih,1:detectors[b]-1,is,rsr,1])
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

save,/compress,ncollects,rg,rg_1,rg_2,rg_1_error,dL_fit,rrcu,rrnl,F_1,F_2,rad_OBC_scan_x,dn_x,filename=outfile

;*****************************************************************************
;  Ouput gains, rrnl, rrcu, and residuals to txt files.
;*****************************************************************************

mindet=[0,0,0,0,1,0,0,0]

outfile2=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.txt'
outfile3=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.scan_avg.txt'
outfile4=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rg.det_avg.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d)'
fmt_2='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d)'
fmt_3='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d)'

openw,lun,outfile2,/get_lun
printf,lun,'Radiametric gains from calibration equation (with rsr)'
printf,lun,'Band   HAM   Detector   Subset   Gain(warm-up)   Gain(cool-down)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR idet=0, detectors[b]-1 DO BEGIN
            FOR isubset=0, subsets[b]-1 DO BEGIN
                FOR iscan=0, maxscans/2-1 DO BEGIN
                    printf,lun,format=fmt_1,band_name[b],ih,idet,isubset,iscan,1/rg_1[b,ih,idet,is,iscan,1,1,0],1/rg_1[b,ih,idet,is,iscan,1,7,0]
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile3,/get_lun
printf,lun,'Radiametric gains from calibration equation (scan averaged with rsr)'
printf,lun,'Band   HAM   Subset   Detector   Gain(warm-up)   Gain(cool-down)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR isubset=0, subsets[b]-1 DO BEGIN
            FOR idet=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_2,band_name[b],ih,isubset,idet,1/mean(rg_1[b,ih,idet,isubset,0:maxscans/2-1,1,1,0]), $
                  1/mean(rg_1[b,ih,idet,isubset,0:maxscans/2-1,1,7,0])
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile4,/get_lun
printf,lun,'Radiametric gains from calibration equation (detector and scan averaged with rsr)'
printf,lun,'Band   HAM   Subset   Gain(warm-up)   Gain(cool-down)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR isubset=0, subsets[b]-1 DO BEGIN
            printf,lun,format=fmt_3,band_name[b],ih,isubset,1/mean(rg_1[b,ih,mindet[b]:detectors[b]-1,isubset,0:maxscans/2-1,1,1,0]), $
              1/mean(rg_1[b,ih,mindet[b]:detectors[b]-1,isubset,0:maxscans/2-1,1,7,0])
        ENDFOR
    ENDFOR
ENDFOR

outfile5=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.residual.warm-up.txt'
outfile6=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.residual.cool-down.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile5,/get_lun
printf,lun,'Fitting Residuals from calibration equation (scan averaged with rsr)'
printf,lun,'BAND   COLLECT   HAM   SUBSET   DETECTOR   dn   dL_OBC   RESIDUAL(FRACTIONAL)   RESIDUAL'
FOR b=0, nbands-1 DO BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        xx=WHERE(warm[*] EQ col)
        IF xx EQ -1 THEN continue
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    printf,lun,format=fmt_1,band_name[b],col,ih,is,id+1,mean(dn_x[col,b,ih,id,is,0:maxscans/2-1]), $
                      mean(rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1,0]),mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,1,0]), $
                      mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,1,0]*rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1,0])
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

openw,lun,outfile6,/get_lun
printf,lun,'Fitting Residuals from calibration equation (scan averaged with rsr)'
printf,lun,'BAND   COLLECT   HAM   SUBSET   DETECTOR   dn   dL_OBC   RESIDUAL(FRACTIONAL)   RESIDUAL'
FOR b=0, nbands-1 DO BEGIN
    FOR col=0, ncollects-1 DO BEGIN
        xx=WHERE(warm[*] EQ col)
        IF xx EQ -1 THEN continue
        FOR ih=0, nham-1 DO BEGIN
            FOR is=0, subsets[b]-1 DO BEGIN
                FOR id=0, detectors[b]-1 DO BEGIN
                    printf,lun,format=fmt_1,band_name[b],col,ih,is,id+1,mean(dn_x[col,b,ih,id,is,0:maxscans/2-1]), $
                      mean(rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1,0]),mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,7,0]), $
                      mean(dL_fit[col,b,ih,id,is,0:maxscans/2-1,7,0]*rad_OBC_scan_x[col,b,ih,id,is,0:maxscans/2-1,0])
                ENDFOR
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

outfile7=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.coeff.warm-up.txt'
outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.coeff.cool-down.txt'

fmt_1='(a5, 5x, i2, 5x, i2, 5x, i2, 5x, d, 5x, d, 5x, d)'

openw,lun,outfile7,/get_lun
printf,lun,'Radiametric coefficients for warm-up (scan averaged with rsr)'
printf,lun,'BAND   HAM   SUBSET   DETECTOR   a0   a1   a2'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_1,band_name[b],ih,is,id+1,mean(rg_1[b,ih,id,is,0:maxscans/2-1,0,1,0]), $
                  mean(rg_1[b,ih,id,is,0:maxscans/2-1,1,1,0]),mean(rg_1[b,ih,id,is,0:maxscans/2-1,2,1,0]),mean(rg_1_error[b,ih,id,is,0:maxscans/2-1,0,1,0]), $
                  mean(rg_1_error[b,ih,id,is,0:maxscans/2-1,1,1,0]),mean(rg_1_error[b,ih,id,is,0:maxscans/2-1,2,1,0])
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR

openw,lun,outfile8,/get_lun
printf,lun,'Radiametric coefficients for cool-down (scan averaged with rsr)'
printf,lun,'BAND   HAM   SUBSET   DETECTOR   a0   a1   a2   error(a0   a1   a2)'
FOR b=0, nbands-1 DO BEGIN
    FOR ih=0, nham-1 DO BEGIN
        FOR is=0, subsets[b]-1 DO BEGIN
            FOR id=0, detectors[b]-1 DO BEGIN
                printf,lun,format=fmt_1,band_name[b],ih,is,id+1,mean(rg_1[b,ih,id,is,0:maxscans/2-1,0,7,0]), $
                  mean(rg_1[b,ih,id,is,0:maxscans/2-1,1,7,0]),mean(rg_1[b,ih,id,is,0:maxscans/2-1,2,7,0]),mean(rg_1_error[b,ih,id,is,0:maxscans/2-1,0,7,0]), $
                  mean(rg_1_error[b,ih,id,is,0:maxscans/2-1,1,7,0]),mean(rg_1_error[b,ih,id,is,0:maxscans/2-1,2,7,0])
            ENDFOR
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

outfile8=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.max.txt'
outfile9=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrcu.avg.txt'
outfile10=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.max.txt'
outfile11=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.rrnl.avg.txt'

fmt_2='(a5, 5x, i2, 5x, i2, 5x, d, 5x, d)'

openw,lun,outfile8,/get_lun
printf,lun,'RRCU (worst case detector with RSR)'
printf,lun,'Band  HAM  SUBSET   RRCU(warm-up)  RRCU(cool-down)'
FOR b=0, nbands-1 DO BEGIN
    FOR is=0, subsets[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name[b],ih,is,max(rrcu[b,ih,0:detectors[b]-1,is,0,0]),max(rrcu[b,ih,0:detectors[b]-1,is,0,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun
		
openw,lun,outfile9,/get_lun
printf,lun,'RRCU (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET   RRCU(warm-up)  RRCU(cool-down)'
FOR b=0, nbands-1 DO BEGIN
    FOR is=0, subsets[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name[b],ih,is,mean(rrcu[b,ih,0:detectors[b]-1,is,0,0]),mean(rrcu[b,ih,0:detectors[b]-1,is,0,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile10,/get_lun
printf,lun,'RRNL (worst case detector with RSR)'
printf,lun,'Band  HAM  SUBSET   RRNL(warm-up)  RRNL(cool-down)'
FOR b=0, nbands-1 DO BEGIN
    FOR is=0, subsets[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name[b],ih,is,max(rrnl[b,ih,0:detectors[b]-1,is,0,0]),max(rrnl[b,ih,0:detectors[b]-1,is,0,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

openw,lun,outfile11,/get_lun
printf,lun,'RRNL (detector averaged with RSR)'
printf,lun,'Band  HAM  SUBSET   RRNL(warm-up)  RRNL(cool-down)'
FOR b=0, nbands-1 DO BEGIN
    FOR is=0, subsets[b]-1 DO BEGIN
        FOR ih=0, nham-1 DO BEGIN
            printf,lun,format=fmt_2,band_name[b],ih,is,mean(rrnl[b,ih,0:detectors[b]-1,is,0,0]),mean(rrnl[b,ih,0:detectors[b]-1,is,0,1])
        ENDFOR
    ENDFOR
ENDFOR
free_lun,lun

outfile9=outpath+a2[a3-3]+'.'+a2[a3-2]+'.'+a2[a3-1]+'.temporary.txt'

openw,lun,outfile9,/get_lun
FOR b=0, nbands-1 DO BEGIN
    FOR is=0, subsets[b]-1 DO BEGIN
        temp1=0
        FOR id=0, detectors[b]-1 DO BEGIN
            
            IF id/2-id/2.+.5*1 NE 0 THEN continue
            temp1=temp1+mean(rg_1[b,0,id,is,0:maxscans/2-1,1,4])
        ENDFOR
        printf,lun,temp1/(detectors[b]/2)
    ENDFOR
ENDFOR
free_lun,lun
close,lun,/file,/all

END
